

/************************************************************************/
/*                                                                      */
/*    WebServer                                                         */
/*                                                                      */
/*    A Example chipKIT HTTP Server implementation                      */
/*    This sketch is designed to work with web browser clients          */
/*                                                                      */
/************************************************************************/
/*    Author:     Keith Vogel                                           */
/*    Copyright 2014, Digilent Inc.                                     */
/************************************************************************/
/* 
*
* Copyright (c) 2013-2014, Digilent <www.digilentinc.com>
* Contact Digilent for the latest version.
*
* This program is free software; distributed under the terms of 
* BSD 3-clause license ("Revised BSD License", "New BSD License", or "Modified BSD License")
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* 1.    Redistributions of source code must retain the above copyright notice, this
*        list of conditions and the following disclaimer.
* 2.    Redistributions in binary form must reproduce the above copyright notice,
*        this list of conditions and the following disclaimer in the documentation
*        and/or other materials provided with the distribution.
* 3.    Neither the name(s) of the above-listed copyright holder(s) nor the names
*        of its contributors may be used to endorse or promote products derived
*        from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/************************************************************************/
/*  Revision History:                                                   */
/*                                                                      */
/*    6/15/2014(KeithV): Created                                        */
/************************************************************************/
/************************************************************************/
/*                                                                      */
/*  This HTTP Server Example contains sample content in the content     */
/*  subdirectory under the sketch directory. The files in the content   */
/*  directory should be copied to the root of your SD card. However,    */
/*  you can view the content by opening HomePage.htm page               */
/*  with your browser right out of the content subdirectory before      */
/*  loading it on the SD card. The sample pages contain                 */
/*  instructions on how to set up this example and therefore may be     */
/*  valuable to read before proceeding.                                 */
/*                                                                      */
/************************************************************************/


//************************************************************************
//************************************************************************
//********  SET THESE LIBRARIES FOR YOUR HARDWARE CONFIGURATION **********
//************************************************************************
//************************************************************************

/************************************************************************/
/*                                                                      */
/*    Network Hardware libraries                                        */
/*    INCLUDE ONLY ONE                                                  */
/*                                                                      */
/************************************************************************/
// You MUST select 1 and ONLY 1 of the following hardware libraries
// they are here so that MPIDE will put the lib path on the compiler include path.
#include <MRF24G.h>                         // This is for the MRF24WGxx on a pmodWiFi or WiFiShield
//#include <IM8720PHY.h>                      // This is for the the Internal MAC and SMSC 8720 PHY

/************************************************************************/
/*    Network libraries                                                 */
/************************************************************************/
// The base network library
// this is a required library
// Do not comment out this library
#include <DEIPcK.h>

//  -----  COMMENT THIS OUT IF YOU ARE NOT USING WIFI  -----
#include <DEWFcK.h>

//************************************************************************
//************************************************************************
//**************** END OF LIBRARY CONFIGURATION **************************
//************************************************************************
//************************************************************************

/************************************************************************/
/*                                                                      */
/*           YOU MUST.....                                              */
/*                                                                      */
/*    You MUST put HTTPServerConfig.h in your sketch director           */
/*    And you MUST configure it with your network parameters            */
/*                                                                      */
/*    You also MUST load your content onto your SD card and             */
/*    the file HomePage.htm MUST exist at the root of the SD            */
/*    file structure. Of course you must insert your SD card            */
/*    into the SD reader on the chipKIT board                           */
/*                                                                      */
/*    Go do this now....                                                */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*    Other libraries; Required libraries                               */
/************************************************************************/
// You must have an SD card reader somewhere
// as the HTTP server uses the SD card to hold the HTML pages.
// this is a required library; we must specify this in the .pde for
// MPIDE to include it.
#include    <SD.h>

//this library is needed to control the servos on the mount
#include <Servo.h>

// and this is the HTTPServer library code.
// this is a required library
#include    <HTTPServer.h>



/************************************************************************/
/*    HTTP URL Matching Strings                                         */
/************************************************************************/
// These are the HTTP URL match strings for the dynamically created
// HTML rendering functions.
// Make these static const so they get put in flash
static const char szHTMLRestart[]       = "GET /Restart ";
static const char szHTMLTerminate[]     = "GET /Terminate ";
static const char szHTMLReboot[]        = "GET /Reboot ";
static const char szHTMLFavicon[]       = "GET /favicon.ico ";
static const char szHTMLGetPins[]       = "GET /PinsPage.htm ";
static const char szHTMLPostPins[]      = "POST /PinsPage.htm ";

// here is our sample/example dynamically created HTML page
GCMD::ACTION ComposeHTMLGetPINS(CLIENTINFO * pClientInfo);
GCMD::ACTION ComposeHTMLPostPINS(CLIENTINFO * pClientInfo);



// get rid of as much of the heap as we can, the SD library requires some heap
#define CHANGE_HEAP_SIZE(size) __asm__ volatile ("\t.globl _min_heap_size\n\t.equ _min_heap_size, " #size "\n")
CHANGE_HEAP_SIZE(0x200);




// Create servos here
Servo pan;
Servo tilt;
Servo height;

//create count
int count=0;

//global servo positions
int pan_position;
int tilt_position;


//GLOBAL previous pin states
#define pin2 2
#define pin3 3
#define pin4 4
#define pin5 5
#define pin6 6
#define pin7 7
#define pin8 8
#define pin9 9
#define pin10 10
#define top_limit_pin 39
#define bottom_limit_pin 38

int tilt_up_previous_state;
int tilt_down_previous_state;
int pan_right_previous_state;
int pan_left_previous_state;
int height_up_previous_state;
int height_down_previous_state;








/***    void setup(void)
 *
 *    Parameters:
 *          None
 *              
 *    Return Values:
 *          None
 *
 *    Description: 
 *    
 *      Arduino Master Initialization routine
 *      
 *      
 * ------------------------------------------------------------ */
void setup(void) 
{   
    // Must do a Serial.begin because the HTTP Server
    // has diagnostic prints in it.
    Serial.begin(9600);
    Serial.println("Mount Control v1.0");
 

    InitializePins();

    // add rendering functions for dynamically created web pages
    // max of 10 AddHTMLPage() allowed 

    // these are the Select picture pages in Post.htm
    AddHTMLPage(szHTMLGetPins,      ComposeHTMLGetPINS);
    AddHTMLPage(szHTMLPostPins,     ComposeHTMLPostPINS);

    // comment this out if you do not want to support
    // restarting the network stack from a browser
    AddHTMLPage(szHTMLRestart,      ComposeHTMLRestartPage);

    // comment this out if you do not want to support
    // terminating the server from a browser
    AddHTMLPage(szHTMLTerminate,    ComposeHTMLTerminatePage);

    // comment this out if you do not want to support
    // rebooting (effectively hitting MCLR) the server from a browser
    AddHTMLPage(szHTMLReboot,       ComposeHTMLRebootPage);

    // This example supports favorite ICONs, 
    // those are those icon's next to the URL in the address line 
    // on the browser once the page is displayed.
    // To support those icons, have at the root of the SD file direcotory
    // an ICON (.ico) file with your ICON in it. The file MUST be named
    // favicon.ico. If you do not have an icon, then uncomment the following
    // line so the server will tell the browser with an HTTP file not found
    // error that we don't have a favoite ICON.
    // AddHTMLPage(szHTMLFavicon,      ComposeHTTP404Error);

    // Make reading files from the SD card the default compose function
    SetDefaultHTMLPage(ComposeHTMLSDPage);

    // Initialize the SD card
    SDSetup();

    // Initialize the HTTP server
    ServerSetup();
    
    //setup limit switch pins
    pinMode(top_limit_pin,INPUT);
    pinMode(bottom_limit_pin, INPUT);
    
    
    
    //place servos on the correct pins
    tilt.attach(30);
    tilt.write(90); //make sure centered
    pan.attach(31);
    pan.write(90); 
    height.attach(32);
    height.write(93); //make sure continuous rotation servo is not moving
    
    //makes sure design at the bottom limit switch (pin 38 -> high)
  //  while(digitalRead(bottom_limit_pin)==HIGH)
  //  {
      //move servo down until at bottom switch
 //     height.write(40); // might have to change this direction based on how mount is constructed
 //     delay(100); //delay a little bit
 //   }
 
    //stop height servo again
    height.write(93);
    

    
    //initialize states of pins
    tilt_up_previous_state=LOW;
    tilt_down_previous_state=LOW;
    pan_right_previous_state=LOW;
    pan_left_previous_state=LOW;
    height_up_previous_state=LOW;
    height_down_previous_state=LOW;
    
}

/***    void loop(void) 
 *
 *    Parameters:
 *          None
 *              
 *    Return Values:
 *          None
 *
 *    Description: 
 *    
 *      Arduino Master Loop routine
 *      
 *      
 * ------------------------------------------------------------ */
void loop(void) 
{
    // process the HTTP Server
    ProcessServer();
    
    //move servos based on pin states, update only every 600 times through loop though
    if(count==2000) 
    {
     updateServoPositions();
     count=0;
     //Serial.println("Updated");
    }
    count++;
   
    
}


//this function looks at the state of pins and from the defined values, determines which position to put the servos in. 
void updateServoPositions()
{  
  
  int tilt_up=LOW;
  int tilt_down=LOW;
  int pan_right=LOW;
  int pan_left=LOW;
  int height_up=LOW;
  int height_down=LOW;
  
  tilt_up=digitalRead(2);
  tilt_down=digitalRead(3);
  pan_right=digitalRead(5); //check this
  pan_left=digitalRead(4); //check this too
  height_up=digitalRead(6); //check
  height_down=digitalRead(7);
  
  //Serial.println(digitalRead(2));

 
 //####################### tilt control #######################################
  if(tilt_up!=tilt_up_previous_state) //up arrow key hit, want to tilt upwards
  {
    // must be a change, thus increment
    tilt_position=tilt_position + 20; //moves 20 degrees upward
    if(tilt_position>179)
    {
      tilt_position=179; //ensures design not over max
    }
    tilt.write(tilt_position); 
    tilt_up_previous_state=tilt_up; //update previous state
  }
  
  else if(tilt_down!=tilt_down_previous_state) //down arrow key hit, want to tilt downwards
  {
    // must be a change, thus increment
    tilt_position=tilt_position - 20; //moves 20 degrees upward
    if(tilt_position<1)
    {
      tilt_position=5; //ensures design not over max
    }
    tilt.write(tilt_position); 
    tilt_down_previous_state=tilt_down; //update previous state
  }
  //####################### END tilt control #######################################
  

  
  //######################################### pan control ##################################################
  
  if(pan_right!=pan_right_previous_state) //right arrow key hit, want to pan right
  {
    // must be a change, thus increment
    pan_position=pan_position + 20; //moves 20 degrees upward
    if(pan_position>179)
    {
      pan_position=179; //ensures design not over max
    }
    pan.write(pan_position); 
    pan_right_previous_state=pan_right; //update previous state
  }
  
  if(pan_left!=pan_left_previous_state) //left arrow key hit, want to pan left
  {
    // must be a change, thus increment
    pan_position=pan_position - 20; //moves 20 degrees upward
    if(pan_position<1)
    {
      pan_position=5; //ensures design not over max
    }
    pan.write(pan_position); 
    pan_left_previous_state=pan_left; //update previous state
  }
  // ############################################# end PAN control #################################
  
  
  
  //####################################### start height control ###################################
if(height_up!=height_up_previous_state) //8 key hit, want to go 
  {
    // must be a change, thus move up
    height.write(180);
     int up_count=0;
     while((up_count<10) ) //&& (digitalRead(top_limit_pin)==LOW)
     {
       delay(100);
       up_count++;
     }
     
     height.write(93); // stop servo
     
    height_up_previous_state=height_up; //update previous state
  }
  
  if(height_down!=height_down_previous_state) //2 arrow key hit, want to go down
  {
      height.write(0);
     int down_count=0;
     while((down_count<10) ) // && (digitalRead(bottom_limit_pin)==LOW)
     {
       delay(100);
       down_count++;
     }
     
     height.write(93); // stop servo
    
    height_down_previous_state=height_down; //update previous state
  }
  
  //####################################### end height control ###################################
  
  else //all states (previous and current at digital low
  {
    //do nothing
  }
  
  
}
